from zope import schema
from zope.interface import Interface

from zope.app.container.constraints import contains
from zope.app.container.constraints import containers

from Products.humanator import humanatorMessageFactory as _

# -*- extra stuff goes here -*-

class IHumanatorfolder(Interface):
    """Humanator folder"""


class IHumanatorquestion(Interface):
    """Humanator question"""




"""Define a browser view for the 'Todo folder' content type. In the FTI 
configured in profiles/default/types/*.xml, this is being set as the default
view of that content type.
"""

from Acquisition import aq_inner

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.app.content.browser.foldercontents import FolderContentsView

class Humanator_questionView(FolderContentsView):
    """Default view of a Todo folder
    """

    __call__ = ViewPageTemplateFile('humanatorquestion_view.pt')



from zope.interface import Interface

class IHumanatorView(Interface):
    """Human style question generating and verifying view
        
    Usage: 
        
        - Use the view from a page to generate a question.
                
        - The user will answer the question
        
        - Use the user input to verify.
        

        
    """
    
    def question(self):
        """Generate a question"""
    
    
    def verify(self, input):
        """Verify the user-supplied input.
        
        Returns a boolean value indicating if the input matched
        
        """



"""Define a browser view for the 'Todo folder' content type. In the FTI 
configured in profiles/default/types/*.xml, this is being set as the default
view of that content type.
"""

from Acquisition import aq_inner

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.app.content.browser.foldercontents import FolderContentsView

class Humanator_folderView(FolderContentsView):
    """Default view of a Humanator folder
    """

    __call__ = ViewPageTemplateFile('humanatorfolder_view.pt')


    def folder_listing(self):
        results = ''
        for i in self.context.objectIds():
            obj = self.context.unrestrictedTraverse('/'.join(self.context.getPhysicalPath()) + '/' + i)
            url = obj.absolute_url()
            title = obj.Title()
            answer = obj.getAnswer()           
            wftool = self.context.portal_workflow
            state = wftool.getInfoFor(obj, 'review_state')
            if state == 'published':
                results += ("<li><p><a href='%s'>%s</a></p><p>%s</p></li>" % (url,title, answer))
            else:
                results += ("<li><p><a href='%s'><strike>%s</strike></a></p><p><strike>%s</strike></p></li>" %
                        (url,title, answer))
        return results 



"""Common configuration constants
"""

PROJECTNAME = 'Products.humanator'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'Humanator question': 'Products.humanator: Add Humanator question',
    'Humanator folder': 'Products.humanator: Add Humanator folder',
}


# question and verifier generation
# TODO: replace ValueErrors with custom errors

from sys import stderr
import md5

import os.path
import random
import sys


from interfaces import IHumanatorView
from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from zope.interface import implements

class NoHumanatorQuestionsError(Exception):
    """ Raised when no humanator questions have been created/published """

class HumanatorQuestionsDoesNotExist(Exception):
    """ Raised when no humanator question by that id is requested """

class BogusQuestionCheckError(Exception):
    """ Raised when XSS attempts are made."""



class Humanator(BrowserView):
    implements(IHumanatorView)

    _session_id = None
    __name__ = 'humanator'

        
    def question(self):
        pc = getToolByName(self.context, 'portal_catalog')
        results = pc(portal_type='Humanator question', review_state='published')
        if results:
            question = random.sample(results,1)[0]
            question = question.getObject()
        else:
            raise NoHumanatorQuestionsError

        return question

    def verify(self, input):

        # fetch the question
        pc = getToolByName(self.context, 'portal_catalog')
        results = pc(portal_type='Humanator question', review_state='published', id = self.request.form['question_id'])
        if results:
            question = results[0].getObject()
        else:
            # replace this value error with something more specific            
            raise HumanatorQuestionsDoesNotExist
            
        # Is this a valid response or is there a faked response attempted?   
        if md5.new(question.Title()).hexdigest() != self.request.form['id_check']:
            # replace this value error with something more specific
            raise BogusQuestionCheckError

        # now lets see if the input equals the answer
        if input == question.getAnswer():
            return True
            
        
        return False





import md5

from zope.component import getMultiAdapter
from zope.app.form.browser import ASCIIWidget
from zope.app.form.interfaces import ConversionError
from zope.app.form.browser.textwidgets import renderElement
from zope.i18n import MessageFactory

from Acquisition import aq_inner


_ = MessageFactory('Products.humanator')




class HumanatorWidget(ASCIIWidget):
    def __call__(self):
        humanator = getMultiAdapter((aq_inner(self.context.context), self.request), name='humanator')
        kwargs = {'type': self.type,
                  'name': self.name,
                  'id': self.name,
                  'cssClass': self.cssClass,
                  'style': self.style,
                  'size': self.displayWidth,
                  'extra': self.extra}
        if self.displayMaxWidth:
            kwargs['maxlength'] = self.displayMaxWidth # TODO This is untested.
            
        question = humanator.question()

        text = u"""<div class="humanatorQuestion"><p>%(question)s</p>
        <input type="hidden" name="question_id" value="%(id)s" />
        <input type="hidden" name="id_check" value="%(id_check)s" />
        %(element)s
        """ % {'question':question.Title(), 
                'element':renderElement(self.tag, **kwargs), 
                'id':question.id, 
                'id_check':md5.new(question.Title()).hexdigest()}
        return text
         
    def _toFieldValue(self, input):
        # Verify the user input against the question
        humanator = getMultiAdapter((aq_inner(self.context.context), self.request), name='humanator')
        if not humanator.verify(input):
            raise ConversionError(_(u'The answer you entered was wrong, please enter the new one.'))
        return super(HumanatorWidget, self)._toFieldValue(input)


"""Definition of the humanator folder content type
"""

from zope.interface import implements, directlyProvides

from Products.Archetypes import atapi
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata

from Products.humanator import humanatorMessageFactory as _
from Products.humanator.interfaces import IHumanatorfolder
from Products.humanator.config import PROJECTNAME

HumanatorfolderSchema = folder.ATFolderSchema.copy() + atapi.Schema((

    # -*- Your Archetypes field definitions here ... -*-

))

# Set storage on fields copied from ATFolderSchema, making sure
# they work well with the python bridge properties.

HumanatorfolderSchema['title'].storage = atapi.AnnotationStorage()
HumanatorfolderSchema['title'].widget.label = 'Name'
HumanatorfolderSchema['description'].storage = atapi.AnnotationStorage()
HumanatorfolderSchema['description'].widget.description = 'A short summary of the humanator folder.'

schemata.finalizeATCTSchema(HumanatorfolderSchema, folderish=True, moveDiscussion=False)

class Humanatorfolder(folder.ATFolder):
    """A folder that contains humanator questions"""
    implements(IHumanatorfolder)

    portal_type = "Humanator folder"
    meta_type = "Humanator folder"
    schema = HumanatorfolderSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

atapi.registerType(Humanatorfolder, PROJECTNAME)



"""Definition of the Humanator question content type
"""

from zope.interface import implements, directlyProvides

from Products.Archetypes import atapi
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata

from Products.humanator import humanatorMessageFactory as _
from Products.humanator.interfaces import IHumanatorquestion
from Products.humanator.config import PROJECTNAME

HumanatorquestionSchema = folder.ATFolderSchema.copy() + atapi.Schema((


    atapi.StringField(
        name='answer',
        storage = atapi.AnnotationStorage(),
        required=True,
        widget=atapi.StringWidget(
            description=_(u"Notes for this todo question."),
        ),
    ),

))

# Set storage on fields copied from ATContentTypeSchema, making sure
# they work well with the python bridge properties.

HumanatorquestionSchema['title'].storage = atapi.AnnotationStorage()
HumanatorquestionSchema['title'].widget.label = 'Question'
HumanatorquestionSchema['description'].storage = atapi.AnnotationStorage()
HumanatorquestionSchema['description'].widget.visible = {'edit': 'hidden', 'view': 'invisible'}

schemata.finalizeATCTSchema(HumanatorquestionSchema, moveDiscussion=False)

class Humanatorquestion(folder.ATFolder):
    """Description of the Humanator questionSchema Type"""
    implements(IHumanatorquestion)

    portal_type = "Humanator question"
    meta_type = "Humanator question"
    schema = HumanatorquestionSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

atapi.registerType(Humanatorquestion, PROJECTNAME)
